@@ -1,6 +1,6 @@
-#!/usr/local/bin/python2.5
+#!/usr/local/bin/python2.7
 
-# SVN Commits Mailing List Parser
+# Commits Mailing List Parser
 # For generating weekly commit digests
 # (c) John Stowers
 # Public Domain
@@ -94,10 +94,14 @@ class _HtmlRenderer:
         #template back to system dirs e.g. /usr/
         self.template = htmltmpl.TemplateManager(precompile=0, debug=0).prepare(template_name)
         self.tproc = htmltmpl.TemplateProcessor()
-        self.tproc.set("date_generated", datetime.date.today().strftime("%Y-%B"))
+        #do not use UTC here, this is user displayed, local timezon
+        self.tproc.set("date_generated", datetime.datetime.now().strftime("%Y-%B"))
         self.tproc.set("page_name", page_name)
 
-    def render_template(self):
+    def render_variable(self, name, value):
+        self.tproc.set(name, value)
+
+    def render_template(self, *args, **kwargs):
         pass
 
     def render(self):
@@ -112,9 +116,11 @@ class SummaryHtmlRenderer(_HtmlRenderer):
 
     SECTION_PROJECT = "projects"
     SECTION_AUTHOR = "authors"
+    SECTION_TODAY_DATE = "today_date"
+    SECTION_LAST_DATE = "last_date"
 
     def __init__(self):
-        _HtmlRenderer.__init__(self, os.path.join(DATADIR,"summary.tmpl"), "GNOME Development Activity Summary")
+        _HtmlRenderer.__init__(self, os.path.join(DATADIR,"summary.tmpl"), "GNOME Development Activity")
         self._data = {}
 
     def _get_chart_url(self, section, data_name, data_data, width=500,limit=20, bh=20):
@@ -171,9 +177,9 @@ class SummaryHtmlRenderer(_HtmlRenderer):
         except KeyError:
             return []
 
-class SVNCommitsParser(sgmllib.SGMLParser):
+class CommitsMailParser(sgmllib.SGMLParser):
     """
-    Parses svn-commits, looking for strings of the form
+    Parses commits, looking for strings of the form
     <li><a name="01439" href="msg01439.html">gtk+ r21606 - in trunk: . gtk</a>&nbsp;&nbsp;cdywan</li>
     """
     def __init__(self, verbose=0):
@@ -186,6 +192,7 @@ class SVNCommitsParser(sgmllib.SGMLParser):
         self.inside_a_element = 0
         self.inside_li_element = 0
         self.inside_strong_element = 0
+        self.commit_number = 0
 
     def start_li(self, attributes):
         self.inside_li_element = 1
@@ -227,27 +234,48 @@ class SVNCommitsParser(sgmllib.SGMLParser):
             elif self.msg in ("Home", "News", "Projects", "Art", "Support", "Development", "Community", "List archives", "Thread", "Author"):
                 pass
             else:
-                self.updates.append( (self.msg, self.author, self.date) )
+                #we dont get the exact time of the commit, so inorder to make sorting etc be
+                #correct, add 1 second to the date for each line parsed on the page (as the
+                #most recent commits are at the bottom of the page)
+                self.updates.append( (self.msg, self.author, self.date + datetime.timedelta(0, self.commit_number)) )
+                self.commit_number += 1
 
     def get_num_parsed_lines(self):
         return len(self.updates)
 
-class Stats:
+class Stats(threading.Thread, gobject.GObject):
 
-    RE_EXP = "^\[([\w+\-/]+)\] (.*)"
-    RE_TRANSLATION_MESSAGE = ".*([Tt]ranslation|[Tt]ranslations]|[Ll]anguage).*"
-    LIST_ARCHIVE_URL = "http://mail.gnome.org/archives/commits-list/%s/date.html"
+    RE_EXP = "^\[([\w+\-\./]+)(: .*)?\] (.*)"
+    RE_TRANSLATION_MESSAGE = ".*([Tt]ranslation|[Tt]ranslations]|[Ll]anguage|[Dd]amned [Ll]ies|[Hh]indi).*"
+    LIST_ARCHIVE_URL = "https://mail.gnome.org/archives/commits-list/%s/date.html"
+
+    ALL_PROJECTS_URL = "http://git.gnome.org/repositories.txt"
 
     TRANSLATION_INCLUDE = "include"
     TRANSLATION_EXCLUDE = "exclude"
     TRANSLATION_ONLY = "only"
     TRANSLATION_CHOICES = (TRANSLATION_INCLUDE,TRANSLATION_EXCLUDE,TRANSLATION_ONLY)
 
-    def __init__(self, filename, days, translations):
+    __gsignals__ = {
+        "completed": (
+            gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
+    }
+
+    def __init__(self, filename, days, translations, includeall):
+
+        threading.Thread.__init__(self)
+        gobject.GObject.__init__(self)
+
         self.days = days
         self.filename = filename
+        self.includeall = includeall
 
-        self.projects = []
+        #all the projects on the GNOME git servier, whether they have seen
+        #any commits over the period or not
+        self.allprojects = {}
+        #projects with activity,
+        #   name : [(branch_name, date, freq), ...]
+        self.projects = {}
         #parse stats, (parsed ok, total, num translations)
         self.parse_stats = (0,0,0)
 
@@ -279,41 +307,68 @@ class Stats:
         self.rt = re.compile(self.RE_TRANSLATION_MESSAGE)
         self.rend = SummaryHtmlRenderer()
 
-    def _download_page(self, url):
+        self.todaydate = datetime.datetime.utcnow()
+        self.lastdate = self.todaydate - datetime.timedelta(days=self.days)
+
+    def _download_page(self, url, tries=5):
+        i = 1
         msg = ""
-        try:
-            print "DOWNLOADING PAGE: %s" % url
-            return urllib2.urlopen(urllib2.Request(url))
-        except urllib2.HTTPError, e:
-            msg = "The server couldn\'t fulfill the request. (error code: %s)" % e.code
-        except urllib2.URLError, e:
-            msg = "We failed to reach a server. (reason: %s)" % e.reason
-        except Exception, e:
-            msg = str(e)            
+        while i <= tries:
+            try:
+                print "DOWNLOADING PAGE: %s (attempt %d)" % (url, i)
+                return urllib2.urlopen(urllib2.Request(url))
+            except urllib2.HTTPError, e:
+                msg = "The server couldn\'t fulfill the request. (error code: %s)" % e.code
+            except urllib2.URLError, e:
+                msg = "We failed to reach a server. (reason: %s)" % e.reason
+            except Exception, e:
+                msg = str(e)
+            i += 1
 
         print "COULD NOT DOWNLOAD: %s (%s)" % (url, msg)
         return None
 
+    def _get_archive_url(self, date):
+        #we need to ignore the system locale because the list archive URLS
+        #are in english
+        MONTHS = (
+            "January",
+            "February",
+            "March",
+            "April",
+            "May",
+            "June",
+            "July",
+            "August",
+            "September",
+            "October",
+            "November",
+            "December")
+
+        return self.LIST_ARCHIVE_URL % ("%s-%s" % (
+                            date.year,
+                            MONTHS[date.month - 1]))
+
     def collect_stats(self):
+        if self.includeall:
+            f = self._download_page(self.ALL_PROJECTS_URL)
+            if f:
+                self.allprojects = [l.strip() for l in f.readlines()]
+
         if self.filename and os.path.exists(self.filename):
             files = [
                 (open(self.filename, "r"), self.filename)
             ]
         else:
-            today = datetime.date.today()
-            last = today - datetime.timedelta(days=self.days)
-            files = []
-
-
-            filename = self.LIST_ARCHIVE_URL % today.strftime("%Y-%B")
-            files.append(
+            filename = self._get_archive_url(self.todaydate)
+            files = [
                 (self._download_page(filename), filename)
-            )
+            ]
 
             #if we are in the first n days of this month, and we require > n days of data
             #then also get the last month
-            if today.month != last.month:
-                filename = self.LIST_ARCHIVE_URL % last.strftime("%Y-%B")
+            if self.todaydate.month != self.lastdate.month:
+                filename = self._get_archive_url(self.lastdate)
                 files.append(
                     (self._download_page(filename), filename)
                 )
@@ -327,7 +382,7 @@ class Stats:
             data = f.read()
             f.close()
 
-            parser = SVNCommitsParser()
+            parser = CommitsMailParser()
             parser.parse(data)
 
             fail = []
@@ -339,9 +394,12 @@ class Stats:
 
                 #break up the message and parse
                 try:
-                    proj, message = n.groups()
+                    proj, series, message = n.groups()
                     try:
-                        proj,branch = proj.split("/")
+                        #use maxsplit=1 because some branch names are in the
+                        #form foo/bar, e.g.
+                        #"glib/wip/gapplication"
+                        proj,branch = proj.split("/", 1)
                     except ValueError:
                         branch = "master"
 
@@ -358,6 +416,7 @@ class Stats:
                                 (proj, auth, branch, message, date, istranslation))
 
                 except ValueError:
+                    print msg
                     fail.append(msg)
 
             total = parser.get_num_parsed_lines()
@@ -366,7 +425,7 @@ class Stats:
             self.parse_stats = (parsed,total,numtranslations)
 
             print "PARSING PAGE: %s" % filename
-            print "RESULTS: %s" % self.get_statistics().capitalize()
+            print "RESULTS: %s" % self.get_download_finished_message().capitalize()
 
     def generate_stats(self):
         #Do in 2 steps because my SQL foo is not strong enough to
@@ -408,17 +467,21 @@ class Stats:
         #       page, and hence more recent commits
         i = []
         self.c.execute('''
-                SELECT project, MAX(d) as "d [timestamp]", COUNT(*) as c
+                SELECT project, branch, MAX(d) as "d [timestamp]", COUNT(*) as c
                 FROM commits 
                 WHERE d >= datetime("now","-%d days") 
                 AND istranslation %s 0 
-                GROUP BY project 
+                GROUP BY project, branch
                 ORDER BY d DESC, ROWID DESC''' % (self.days,self.includetranslations))
-        for name, d, freq in self.c:
-            #print "name: %s\n\tdate %s %s\n\t\tfreq: %s" % (name, d, type(d), freq)
+        for name, branch, d, freq in self.c:
             i.append([name, freq, ""])
-            self.projects.append((name, d, freq))
+            try:
+                self.projects[name].append((branch, d, freq))
+            except KeyError:
+                self.projects[name] = [(branch, d, freq)]
 
+        #resort by number of commits
+        i.sort(cmp=lambda x,y: cmp(y[1], x[1]))
         for j in i:
             self.c.execute('''
                     SELECT author
@@ -435,34 +498,74 @@ class Stats:
                     project_name=name, project_freq=freq, project_authors=projects)
 
     def get_summary(self):
+        self.rend.render_variable(
+                self.rend.SECTION_TODAY_DATE,
+                self.todaydate.strftime("%x"))
+        self.rend.render_variable(
+                self.rend.SECTION_LAST_DATE,
+                self.lastdate.strftime("%x"))
         return self.rend.render()
 
     def get_projects(self):
-        return self.projects
+        projects = self.projects
+        if self.includeall:
+            for p in self.allprojects:
+                if p not in projects:
+                    projects[p] = []
+        return projects
 
     def got_data(self):
         return self.parse_stats[0] > 0 and len(self.projects) > 0
 
-    def get_statistics(self):
-        return "matched %d/%d commit messages (%d translations)" % self.parse_stats
+    def get_download_message(self):
+        return "Downloading %d day%s of development history" % (
+                        self.days,
+                        {
+                            True:"s",
+                            False:""
+                        }[self.days > 1])
 
-class UI(threading.Thread):
+    def get_download_finished_message(self):
 
-    BTNS = ("commit_btn","changelog_btn","news_btn","new_patches_btn")
+        def percentage(n,d):
+            return 100.0 * (float(n)/d)
+
+        nmatch,ntotal,ntrans = self.parse_stats
+        msg = "matched %d/%d commit messages (%.2f%%%%), %%s %d translations (%.2f%%%%)" % (
+            nmatch,ntotal,percentage(nmatch,ntotal),ntrans,percentage(ntrans,ntotal))
+        print msg
+        return msg % (
+                    {
+                        self.TRANSLATION_INCLUDE:"including",
+                        self.TRANSLATION_EXCLUDE:"excluding",
+                        self.TRANSLATION_ONLY:"only considering"
+                    }[self.translations])
+
+    def run(self):
+        self.collect_stats()
+        self.generate_stats()
+        gobject.idle_add(gobject.GObject.emit,self,"completed")
+
+class UI:
+
+    BTNS = ("refresh_btn","commit_btn","news_btn","new_patches_btn","new_bugs_btn", "summary_btn")
     CHANGELOG_STR = "http://git.gnome.org/browse/%(project)s/tree/ChangeLog"
     NEWS_STR = "http://git.gnome.org/browse/%(project)s/tree/NEWS"
-    LOG_STR = "http://git.gnome.org/cgit/%(project)s/log"
+    LOG_STR = "http://git.gnome.org/cgit/%(project)s/log?h=%(branch)s"
     NEW_PATCHES_STR = "https://bugzilla.gnome.org/page.cgi?id=patchreport.html&product=%(escaped_project)s&patch-status=&max_days=%(days)s"
+    NEW_BUGS_STR = "https://bugzilla.gnome.org/buglist.cgi?product=%(escaped_project)s&bug_status=UNCONFIRMED&bug_status=NEW&bug_status=ASSIGNED&bug_status=REOPENED&chfield=[Bug creation]&chfieldfrom=%(last_date)s"
 
     #keep in sync with ui file
+    PGO_NOTEBOOK_PAGE = 0
     PROJECT_NOTEBOOK_PAGE = 1
+    SUMMARY_NOTEBOOK_PAGE = 2
 
-    def __init__(self, stats):
-        threading.Thread.__init__(self)
-        self.stats = stats
+    def __init__(self, options):
+        self.options = options
 
         #selected project
         self.project = None
+        self.branch = None
 
         self.builder = _GtkBuilderWrapper(DATADIR, "gnome.ui")
         self.builder.connect_signals(self)
@@ -478,7 +581,7 @@ class UI(threading.Thread):
         self.summaryWebkit = webkit.WebView()
         self.summaryWebkit.load_string(
                         loadingtxt,
-                        "text/html", "iso-8859-15", "commits:")
+                        "text/html", "utf-8", "commits:")
         self.builder.get_object("summaryScrolledWindow").add(self.summaryWebkit)
 
         self.sb = self.builder.get_object("statusbar1")
@@ -486,10 +589,10 @@ class UI(threading.Thread):
         self.projectWebkit = webkit.WebView()
         self.projectWebkit.load_string(
                         loadingtxt,
-                        "text/html", "iso-8859-15", "project:")
+                        "text/html", "utf-8", "project:")
         sw.add(self.projectWebkit)
 
-        self.model = gtk.ListStore(str,int, object)
+        self.model = gtk.TreeStore(str,int, object)
         self.tv = self.builder.get_object("treeview1")
 
         col = gtk.TreeViewColumn("Project Name", gtk.CellRendererText(), text=0)
@@ -502,24 +605,47 @@ class UI(threading.Thread):
 
         rend = gtk.CellRendererText()
         date = gtk.TreeViewColumn("Last Commit", rend)
+        date.set_sort_column_id(2)
         date.set_cell_data_func(rend, self._render_date)
         self.tv.append_column(date)
+        self.model.set_sort_func(2, self._sort_dates)
+        self.model.set_sort_column_id(2, gtk.SORT_ASCENDING)
 
         self.notebook = self.builder.get_object("notebook1")
 
+        #make sure the treeview is visible
+        self.builder.get_object("hpaned1").set_position(350)
+
         w = self.builder.get_object("window1")
         w.show_all()
 
+    def _sort_dates(self, model, iter1, iter2):
+        d1 = model.get_value(iter1, 2)
+        d2 = model.get_value(iter2, 2)
+        if d1 and d2:
+            #newer
+            if d1 > d2:
+                return -1
+            #older
+            elif d2 > d1:
+                return 1
+        #equal
+        return 0
+
     def _render_date(self, column, cell, model, iter_):
         d = model.get_value(iter_, 2)
-        cell.props.text = humanize_date_difference(now=self.time_started, otherdate=d)
+        if d != datetime.datetime.min:
+            cell.props.text = humanize_date_difference(now=self.stats.todaydate, otherdate=d)
+        else:
+            cell.props.text = "unknown"
 
     def _get_details_dict(self):
-        today = datetime.date.today()
+        today = self.stats.todaydate
         old = today-datetime.timedelta(days=self.stats.days)
 
         return {
             "project":self.project,
+            "branch":self.branch,
             "escaped_project":urllib.quote(self.project),
             "today_date":today.strftime("%Y-%m-%d"),
             "last_date":old.strftime("%Y-%m-%d"),
@@ -540,21 +666,31 @@ class UI(threading.Thread):
     def _get_selected_project(self):
         model,iter_ = self.tv.get_selection().get_selected()
         if model and iter_:
-            self.project = model.get_value(iter_, 0)
+            if model.iter_depth(iter_) == 0:
+                #if header is selected use the master branch
+                self.project = model.get_value(iter_, 0)
+                self.branch = "master"
+            else:
+                #otherwise use the shown branch
+                self.branch = model.get_value(iter_, 0)
+                self.project = model.get_value(model.iter_parent(iter_), 0)
         else:
             self._statusbar_update("Please select a project")
             self.project = None
+            self.branch = None
 
         return self.project
 
+    def on_pgo_btn_clicked(self, *args):
+        self.notebook.set_current_page(self.PGO_NOTEBOOK_PAGE)
+
+    def on_summary_btn_clicked(self, *args):
+        self.notebook.set_current_page(self.SUMMARY_NOTEBOOK_PAGE)
+
     def on_commit_btn_clicked(self, *args):
         if self._get_selected_project():
             self._open_project_url(self.LOG_STR % self._get_details_dict())
 
-    def on_changelog_btn_clicked(self, *args):
-        if self._get_selected_project():
-            self._open_project_url(self.CHANGELOG_STR % self._get_details_dict())
-
     def on_news_btn_clicked(self, *args):
         if self._get_selected_project():
             self._open_project_url(self.NEWS_STR % self._get_details_dict())
@@ -563,41 +699,61 @@ class UI(threading.Thread):
         if self._get_selected_project():
             self._open_project_url(self.NEW_PATCHES_STR % self._get_details_dict())
 
+    def on_new_bugs_btn_clicked(self, *args):
+        if self._get_selected_project():
+            self._open_project_url(self.NEW_BUGS_STR % self._get_details_dict())
+
+    def on_refresh_btn_clicked(self, *args):
+        self.refresh()
+
     def on_window1_destroy(self, *args):
         gtk.main_quit()
 
-    def collect_stats_finished(self):
+    def collect_stats_finished(self, stats):
         if not self.stats.got_data():
             self._statusbar_update("Download failed")
             return
 
-        self._statusbar_update("Download finished, %s" % self.stats.get_statistics())
-        for p,d,commits in self.stats.get_projects():
-            self.model.append((p,commits,d))
+        self._statusbar_update("Download finished, %s" % self.stats.get_download_finished_message())
+
+        projects = self.stats.get_projects()
+        for p in projects:
+            newestdate = datetime.datetime.min
+            totalcommits = 0
+            for branch, d, commits in projects[p]:
+                totalcommits += commits
+                newestdate = max(d, newestdate)
+
+            #add the project summary
+            projiter = self.model.append(None, (p,totalcommits,newestdate))
+            #add the branch summary
+            for branch, d, commits in projects[p]:
+                self.model.append(projiter, (branch,commits,d))
+
         self.tv.set_model(self.model)
         for i in self.BTNS:
             self.builder.get_object(i).set_sensitive(True)
 
         self.summaryWebkit.load_string(
                         self.stats.get_summary(), 
-                        "text/html", "iso-8859-15", "commits:"
+                        "text/html", "utf-8", "commits:"
         )
 
-    def run(self):
-        #FIXME: Until we can record the commit time, no need to care about the 
-        #time started, the date will do
-        #self.time_started = datetime.datetime.now()
-        self.time_started = datetime.datetime.fromordinal(datetime.date.today().toordinal())
-        self.stats.collect_stats()
-        self.stats.generate_stats()
-        gobject.idle_add(self.collect_stats_finished)
+    def refresh(self):
+        self.model.clear()
+        for i in self.BTNS:
+            self.builder.get_object(i).set_sensitive(False)
+        self.stats = Stats(
+                        filename=options.source,
+                        days=options.days,
+                        translations=options.translations,
+                        includeall=options.all_projects)
+        self.stats.connect("completed", self.collect_stats_finished)
+        self._statusbar_update(self.stats.get_download_message())
+        self.stats.start()
 
     def main(self):
-        self._statusbar_update("Downloading %d day%s of development history (%s translations)" % (
-                        self.stats.days,
-                        {True:"s",False:""}[self.stats.days > 1],
-                        self.stats.translations))
-        self.start()
+        self.refresh()
         gtk.main()
 
 if __name__ == "__main__":
@@ -607,18 +763,20 @@ if __name__ == "__main__":
     parser.add_option("-s", "--source",
                   help="read statistics from FILE [default: read from web]", metavar="FILE")
     parser.add_option("-d", "--days",
-                  type="int", default=5,
+                  type="int", default=3,
                   help="the number of days to consider for statistics [default: %default]")
     parser.add_option("-t", "--translations",
                   choices=Stats.TRANSLATION_CHOICES,
                   metavar="[%s]" % "|".join(Stats.TRANSLATION_CHOICES),
                   default=Stats.TRANSLATION_EXCLUDE,
                   help="include translation commits in analysis [default: %default]")
+    parser.add_option("-a", "--all-projects",
+                  help="include all GNOME projects, not just those with commits",
+                  action="store_true")
 
     options, args = parser.parse_args()
 
-    s = Stats(filename=options.source, days=options.days, translations=options.translations)
-    ui = UI(s)
+    ui = UI(options)
     ui.main()
 
 
